@@ -265,7 +265,7 @@ def write_posts_to_csv(
     encoding: str = None,
     **kwargs,
 ):
-    """Write posts from an account or group to a CSV file
+    """Write posts from an account or group to a CSV or JSON file
 
     Args:
         account (str): Facebook account name e.g. "nike" or "nintendo"
@@ -281,8 +281,9 @@ def write_posts_to_csv(
     """
     dump_location = kwargs.pop('dump_location', None)
 
-    list_of_posts = list(
-        get_posts(account=account, group=group, remove_source=not bool(dump_location), **kwargs)
+    list_of_posts = utils.safe_consume(
+        get_posts(account=account, group=group, remove_source=not bool(dump_location), **kwargs),
+        kwargs.get("sleep", 0),
     )
 
     if not list_of_posts:
@@ -317,15 +318,25 @@ def write_posts_to_csv(
                 keys.append(key)
 
     if filename is None:
-        filename = str(account or group) + "_posts.csv"
+        if kwargs.get("format") == "json":
+            filename = str(account or group) + "_posts.json"
+        else:
+            filename = str(account or group) + "_posts.csv"
 
     if encoding is None:
         encoding = locale.getpreferredencoding()
 
-    with open(filename, 'w', newline='', encoding=encoding) as output_file:
+    if filename == "-":
+        output_file = sys.stdout
+    else:
+        output_file = open(filename, 'w', newline='', encoding=encoding)
+    if kwargs.get("format") == "json":
+        json.dump(list_of_posts, output_file, default=str, indent=4)
+    else:
         dict_writer = csv.DictWriter(output_file, keys)
         dict_writer.writeheader()
         dict_writer.writerows(list_of_posts)
+    output_file.close()
 
 
 def enable_logging(level=logging.INFO):
