@@ -14,6 +14,9 @@ def run():
     parser.add_argument('account', type=str, help="Facebook account or group")
     parser.add_argument('-f', '--filename', type=str, help="Output filename")
     parser.add_argument('-p', '--pages', type=int, help="Number of pages to download", default=10)
+    parser.add_argument(
+        '-s', '--sleep', type=float, help="How long to sleep for between posts", default=0
+    )
     parser.add_argument(
         '-t',
         '--timeout',
@@ -40,6 +43,13 @@ def run():
         help="Encoding for the output file",
         default=None,
     )
+    parser.add_argument(
+        '-fmt',
+        '--format',
+        type=str.lower,
+        choices=["csv", "json"],
+        help="What format to export as",
+    )
 
     args = parser.parse_args()
 
@@ -47,8 +57,10 @@ def run():
     account_type = 'group' if args.group else 'account'
     kwargs = {
         account_type: args.account,
+        "format": args.format,
         "cookies": args.cookies,
         "timeout": args.timeout,
+        "sleep": args.sleep,
         "options": {
             "reactions": args.reactions,
             "reactors": args.reactors,
