@@ -1,6 +1,6 @@
 import itertools
 import json
-import demjson
+import demjson3 as demjson
 import logging
 import re
 from datetime import datetime
@@ -10,7 +10,7 @@ from urllib.parse import parse_qs, urlparse
 from tqdm.auto import tqdm
 
 from . import utils, exceptions
-from .constants import FB_BASE_URL, FB_MOBILE_BASE_URL
+from .constants import FB_BASE_URL, FB_MOBILE_BASE_URL, FB_W3_BASE_URL
 from .fb_types import Options, Post, RawPost, RequestFunction, Response, URL
 
 
@@ -53,7 +53,6 @@ class PostExtractor:
     )
     comments_regex = re.compile(r'([\d,.KM]+)\s+comment', re.IGNORECASE)
     shares_regex = re.compile(r'([\d,.KM]+)\s+Share', re.IGNORECASE)
-    live_regex = re.compile(r'.+(is live).+')
     link_regex = re.compile(r"href=\"https:\/\/lm\.facebook\.com\/l\.php\?u=(.+?)\&amp;h=")
 
     photo_link = re.compile(r'href=\"(/[^\"]+/photos/[^\"]+?)\"')
@@ -94,6 +93,7 @@ class PostExtractor:
             'post_text': None,
             'shared_text': None,
             'time': None,
+            'timestamp': None,
             'image': None,
             'image_lowquality': None,
             'images': None,
@@ -114,6 +114,7 @@ class PostExtractor:
             'shares': None,
             'post_url': None,
             'link': None,
+            'links': None,
             'user_id': None,
             'username': None,
             'user_url': None,
@@ -131,6 +132,7 @@ class PostExtractor:
             'w3_fb_url': None,
             'reactions': None,
             'reaction_count': None,
+            'with': None,
         }
 
     def extract_post(self) -> Post:
@@ -146,7 +148,7 @@ class PostExtractor:
             self.extract_likes,
             self.extract_comments,
             self.extract_shares,
-            self.extract_link,
+            self.extract_links,
             self.extract_user_id,
             self.extract_username,
             self.extract_video,
@@ -158,6 +160,7 @@ class PostExtractor:
             self.extract_share_information,
             self.extract_availability,
             self.extract_listing,
+            self.extract_with,
         ]
 
         post = self.make_new_post()
@@ -197,10 +200,14 @@ class PostExtractor:
 
         if self.options.get('comments'):
             try:
-                comments = self.extract_comments_full()
-                post.update(comments)
-                if post.get("comments_full") and not post.get("comments"):
-                    post["comments"] = len(post.get("comments_full"))
+                post["comments_full"] = self.extract_comments_full()
+                if self.options.get("comments") != "generator":
+                    # Consume both comment generator and reply generator to return lists
+                    post["comments_full"] = utils.safe_consume(post["comments_full"])
+                    for comment in post["comments_full"]:
+                        comment["replies"] = utils.safe_consume(comment["replies"])
+                    if post.get("comments_full") and not post.get("comments"):
+                        post["comments"] = len(post.get("comments_full"))
 
             except Exception as ex:
                 log_warning("Exception while extracting comments: %r", ex)
@@ -208,7 +215,10 @@ class PostExtractor:
         return post
 
     def extract_post_id(self) -> PartialPost:
-        return {'post_id': self.data_ft.get('top_level_post_id')}
+        return {
+            'post_id': self.live_data.get("ft_ent_identifier")
+            or self.data_ft.get('top_level_post_id')
+        }
 
     def extract_username(self) -> PartialPost:
         elem = self.element.find('h3 strong a,a.actor-link', first=True)
@@ -284,9 +294,7 @@ class PostExtractor:
                 logger.debug(
                     f"Got exact timestamp from publish_time: {datetime.fromtimestamp(timestamp)}"
                 )
-                return {
-                    'time': datetime.fromtimestamp(timestamp),
-                }
+                return {'time': datetime.fromtimestamp(timestamp), 'timestamp': timestamp}
             except (KeyError, ValueError):
                 continue
 
@@ -311,9 +319,7 @@ class PostExtractor:
             logger.debug(
                 f"Got exact timestamp from abbr[data-store]: {datetime.fromtimestamp(time)}"
             )
-            return {
-                'time': datetime.fromtimestamp(time),
-            }
+            return {'time': datetime.fromtimestamp(time), 'timestamp': time}
         except:
             return None
 
@@ -352,11 +358,13 @@ class PostExtractor:
                 result.update({"image": url, "images": [url]})
         return result
 
-    def extract_link(self) -> PartialPost:
-        match = self.link_regex.search(self.element.html)
-        if match:
-            return {'link': utils.unquote(match.groups()[0])}
-        return None
+    def extract_links(self) -> PartialPost:
+        link = self.link_regex.search(self.element.html)
+        if link:
+            link = utils.unquote(link.groups()[0])
+        links = self.element.find(".story_body_container div p a")
+        links = [{"link": a.attrs["href"], "text": a.text} for a in links]
+        return {"link": link, "links": links}
 
     def extract_post_url(self) -> PartialPost:
 
@@ -495,8 +503,15 @@ class PostExtractor:
                     if node["is_playable"]:
                         video_ids.append(node["id"])
                         videos.append(node["playable_url_hd"] or node["playable_url"])
-                    images.append(node["full_width_image"]["uri"])
-                    image_ids.append(node["id"])
+                        images.append(node["full_width_image"]["uri"])
+                        image_ids.append(node["id"])
+                    else:
+                        url = node["url"]
+                        url = url.replace(FB_W3_BASE_URL, FB_MOBILE_BASE_URL)
+                        logger.debug(f"Fetching {url}")
+                        response = self.request(url)
+                        images.append(self.extract_photo_link_HQ(response.text))
+                        image_ids.append(node["id"])
                     descriptions.append(node["accessibility_caption"])
                 return {
                     "image": images[0] if images else None,
@@ -556,7 +571,7 @@ class PostExtractor:
             "image_ids": image_ids,
         }
 
-    def extract_reactions(self) -> PartialPost:
+    def extract_reactions(self, post_id=None) -> PartialPost:
         """Fetch share and reactions information with a existing post obtained by `get_posts`.
         Return a merged post that has some new fields including `reactions`, `w3_fb_url`,
         `fetched_time`, and reactions fields `LIKE`, `ANGER`, `SORRY`, `WOW`, `LOVE`, `HAHA` if
@@ -580,16 +595,33 @@ class PostExtractor:
                 if v["default"]:
                     name = reaction_lookup[k]["display_name"].lower()
                     reactions[name] = v["default"]
+        reaction_count = self.live_data.get("reactioncount")
 
         url = self.post.get('post_url')
-        post_id = self.post.get('post_id')
+        if not post_id:
+            post_id = self.post.get("post_id")
         w3_fb_url = url and utils.urlparse(url)._replace(netloc='www.facebook.com').geturl()
 
         reactors = []
         reactors_opt = self.options.get("reactors")
         if reactors_opt:
             reaction_url = f'https://m.facebook.com/ufi/reaction/profile/browser/?ft_ent_identifier={post_id}'
+            logger.debug(f"Fetching {reaction_url}")
             response = self.request(reaction_url)
+
+            reactions = {}
+            for sigil in response.html.find("span[data-sigil='reaction_profile_sigil']"):
+                k = str(demjson.decode(sigil.attrs.get("data-store"))["reactionType"])
+                v = sigil.find(
+                    "span[data-sigil='reaction_profile_tab_count']", first=True
+                ).text.replace("All ", "")
+                v = utils.convert_numeric_abbr(v)
+                if k == "all":
+                    reaction_count = v
+                elif k in reaction_lookup:
+                    name = reaction_lookup[k]["display_name"].lower()
+                    reactions[name] = v
+
             emoji_class_lookup = {}
             spriteMapCssClass = "sp_E24l_TeOlgh"
             for k, v in self.get_jsmod("UFIReactionIcons").items():
@@ -604,17 +636,16 @@ class PostExtractor:
             if type(reactors_opt) in [int, float] and reactors_opt < limit:
                 limit = reactors_opt
             logger.debug(f"Fetching {limit} reactors")
-            elems = list(
-                response.html.find(
-                    "div#reaction_profile_browser>div,div#reaction_profile_browser1>div"
-                )
-            )
+            elems = list(response.html.find("div[id^='reaction_profile_browser']>div"))
             more = response.html.find("div#reaction_profile_pager a", first=True)
-            if more and limit > 50:
+            while more and len(elems) < limit:
                 url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
-                url = url.replace("limit=50", f"limit={limit - 50}")
                 logger.debug(f"Fetching {url}")
-                response = self.request(url)
+                try:
+                    response = self.request(url)
+                except Exception as e:
+                    logger.error(e)
+                    break
                 prefix_length = len('for (;;);')
                 data = json.loads(response.text[prefix_length:])  # Strip 'for (;;);'
 
@@ -628,6 +659,12 @@ class PostExtractor:
                             'div#reaction_profile_browser>div,div#reaction_profile_browser1>div'
                         )
                         elems.extend(more_elems)
+                    elif action['cmd'] == 'replace':
+                        html = utils.make_html_element(
+                            f"<div id='reaction_profile_browser'>{action['html']}</div>",
+                            url=FB_MOBILE_BASE_URL,
+                        )
+                        more = html.find("div#reaction_profile_pager a", first=True)
             logger.debug(f"Found {len(elems)} reactors")
             for elem in elems:
                 emoji_class = elem.find(f"div>i.{spriteMapCssClass}", first=True).attrs.get(
@@ -649,7 +686,7 @@ class PostExtractor:
             return {
                 'likes': reactions.get("like"),
                 'reactions': reactions,
-                'reaction_count': self.live_data.get("reactioncount"),
+                'reaction_count': reaction_count,
                 'reactors': reactors,
                 'fetched_time': datetime.now(),
                 'w3_fb_url': w3_fb_url,
@@ -705,7 +742,7 @@ class PostExtractor:
 
     def extract_video_lowres(self, video_data_element):
         try:
-            data = json.loads(video_data_element.attrs['data-store'].replace("\\\\", "\\"))
+            data = demjson.decode(video_data_element.attrs['data-store'].replace("\\\\", "\\"))
             return {'video': data.get('src').replace("\\/", "/")}
         except JSONDecodeError as ex:
             logger.error("Error parsing data-store JSON: %r", ex)
@@ -766,7 +803,12 @@ class PostExtractor:
         contentSize = None
         if meta.get("contentSize"):
             contentSize = float(meta['contentSize'].strip("kB")) / 1000
+
+        time = utils.parse_datetime(meta["datePublished"])
+        # Remove the timezone attribute to make it timezone-naive
+        time = time.astimezone().replace(tzinfo=None)
         return {
+            "time": time,
             'video_duration_seconds': utils.parse_duration(meta.get("duration")),
             'video_watches': watches,
             'video_quality': meta.get('videoQuality'),
@@ -777,13 +819,7 @@ class PostExtractor:
 
     def extract_is_live(self):
         header = self.element.find('header')[0].full_text
-
-        match = self.live_regex.search(header)
-
-        if match is not None:
-            return {'is_live': True}
-
-        return {'is_live': False}
+        return {'is_live': "is live" in header, 'was_live': "was live" in header}
 
     def extract_factcheck(self):
         button = self.element.find('button[value="See Why"]', first=True)
@@ -875,6 +911,15 @@ class PostExtractor:
                 if match:
                     image_url = utils.decode_css_url(match.groups()[0])
 
+        reactions = {}
+        if self.options.get("reactions") or self.options.get("reactors"):
+            self.options["reactors"] = True  # Required for comment reaction extraction
+            reactors = comment.find(
+                'a[href^="/ufi/reaction/profile/browser/?ft_ent_identifier="] i', first=True
+            )
+            if reactors:
+                reactions = self.extract_reactions(comment_id)
+
         return {
             "comment_id": comment_id,
             "comment_url": utils.urljoin(FB_BASE_URL, comment_id),
@@ -885,13 +930,68 @@ class PostExtractor:
             "comment_text": text,
             "comment_time": date,
             "comment_image": image_url,
+            "comment_reactors": reactions.get("reactors"),
+            "comment_reactions": reactions.get("reactions"),
+            "comment_reaction_count": reactions.get("reaction_count"),
         }
 
+    def extract_comment_replies(self, replies_url):
+        if not self.options.get("progress"):
+            logger.debug(f"Fetching {replies_url}")
+        try:
+            response = self.request(replies_url)
+        except exceptions.TemporarilyBanned:
+            raise
+        except Exception as e:
+            logger.error(e)
+            return
+        # Skip first element, as it will be this comment itself
+        reply_selector = 'div[data-sigil="comment"]'
+        if self.options.get("noscript"):
+            reply_selector = '#root div[id]'
+        replies = response.html.find(reply_selector)[1:]
+        try:
+            for reply in replies:
+                yield self.parse_comment(reply)
+        except exceptions.TemporarilyBanned:
+            raise
+        except Exception as e:
+            logger.error(f"Unable to parse comment {replies_url} replies {replies}: {e}")
+
+    def extract_comment_with_replies(self, comment):
+        try:
+            result = self.parse_comment(comment)
+            result["replies"] = [
+                self.parse_comment(reply)
+                for reply in comment.find("div[data-sigil='comment inline-reply']")
+            ]
+            replies_url = comment.find(
+                "div.async_elem[data-sigil='replies-see-more'] a[href],div[id*='comment_replies_more'] a[href]",
+                first=True,
+            )
+            if replies_url:
+                reply_generator = self.extract_comment_replies(replies_url.attrs["href"])
+                if result["replies"]:
+                    result["replies"] = itertools.chain(result["replies"], reply_generator)
+                else:
+                    result["replies"] = reply_generator
+            return result
+        except exceptions.TemporarilyBanned:
+            raise
+        except Exception as e:
+            logger.error(f"Unable to parse comment {comment}: {e}")
+
     def extract_comments_full(self):
         """Fetch comments for an existing post obtained by `get_posts`.
         Note that this method may raise multiple http requests per post to get all comments"""
+        if not self.full_post_html:
+            logger.error("Unable to get comments without full post HTML")
+            return
         comments_area_selector = 'div.ufi'
         elem = self.full_post_html.find(comments_area_selector, first=True)
+        if not elem:
+            logger.error("No comments area found")
+            return
         comments_selector = 'div[data-sigil="comment"]'
         if self.options.get("noscript"):
             comments_selector = "div._55wr"
@@ -900,6 +1000,11 @@ class PostExtractor:
             logger.warning("No comments found on page")
             return
 
+        for comment in comments:
+            result = self.extract_comment_with_replies(comment)
+            if result:
+                yield result
+
         more_selector = f"div#see_next_{self.post.get('post_id')} a"
         more = elem.find(more_selector, first=True)
         if not more:
@@ -922,71 +1027,54 @@ class PostExtractor:
             pbar = tqdm(total=limit)
 
         visited_urls = []
-        while more and len(comments) <= limit:
-            url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
-            if url in visited_urls:
+        request_url_callback = self.options.get('comment_request_url_callback')
+        more_url = None
+        if more:
+            more_url = (
+                more.attrs.get("href")
+                + "&m_entstream_source=video_home&player_suborigin=entry_point&player_format=permalink"
+            )
+        if self.options.get("comment_start_url"):
+            more_url = self.options.get("comment_start_url")
+
+        while more_url and len(comments) <= limit:
+            if request_url_callback:
+                request_url_callback(more_url)
+            if more_url in visited_urls:
                 logger.debug("cycle detected, break")
                 break
             if self.options.get("progress"):
                 pbar.update(30)
             else:
-                logger.debug(f"Fetching {url}")
+                logger.debug(f"Fetching {more_url}")
             try:
-                response = self.request(url)
+                response = self.request(more_url)
+            except exceptions.TemporarilyBanned:
+                raise
             except Exception as e:
                 logger.error(e)
                 break
-            visited_urls.append(url)
+            visited_urls.append(more_url)
             elem = response.html.find(comments_area_selector, first=True)
+            if not elem:
+                logger.warning("No comments found on page")
+                break
             more_comments = elem.find(comments_selector)
             if not more_comments:
                 logger.warning("No comments found on page")
                 break
-            comments.extend(more_comments)
+            for comment in more_comments:
+                result = self.extract_comment_with_replies(comment)
+                if result:
+                    yield result
             more = elem.find(more_selector, first=True)
-
-        logger.debug(f"Found {len(comments)} comments")
-        results = []
-        if self.options.get("progress"):
-            pbar.close()
-            pbar = tqdm(total=len(comments))
-        for comment in comments:
-            try:
-                result = self.parse_comment(comment)
-            except Exception as e:
-                logger.error(f"Unable to parse comment {comment}: {e}")
-                continue
-            inline_replies = comment.find("div[data-sigil='comment inline-reply']")
-            result["replies"] = [self.parse_comment(reply) for reply in inline_replies]
-            replies = comment.find(
-                "div.async_elem[data-sigil='replies-see-more'] a[href],div[id*='comment_replies_more'] a[href]",
-                first=True,
-            )
-            if self.options.get("progress"):
-                pbar.update(1)
-            if replies:
-                url = utils.urljoin(FB_MOBILE_BASE_URL, replies.attrs["href"])
-                if not self.options.get("progress"):
-                    logger.debug(f"Fetching {url}")
-                try:
-                    response = self.request(url)
-                except Exception as e:
-                    logger.error(e)
-                    continue
-                # Skip first element, as it will be this comment itself
-                reply_selector = 'div[data-sigil="comment"]'
-                if self.options.get("noscript"):
-                    reply_selector = '#root div[id]'
-                replies = response.html.find(reply_selector)[1:]
-                try:
-                    result["replies"].extend([self.parse_comment(reply) for reply in replies])
-                except Exception as e:
-                    logger.error(
-                        f"Unable to parse comment {result['comment_id']} replies {replies}: {e}"
-                    )
-                    continue
-            results.append(result)
-        return {"comments_full": results}
+            if more:
+                more_url = (
+                    more.attrs.get("href")
+                    + "&m_entstream_source=video_home&player_suborigin=entry_point&player_format=permalink"
+                )
+            else:
+                more_url = None
 
     def parse_share_and_reactions(self, html: str):
         bad_jsons = self.shares_and_reactions_regex.findall(html)
@@ -1004,6 +1092,20 @@ class PostExtractor:
                 "listing_location": divs[2].text,
             }
 
+    def extract_with(self) -> PartialPost:
+        # Header is like "user is with other_user and n others"
+        links = self.element.find("header h3 a")[1:]
+        if links:
+            people = [{"name": links[0].text, "link": links[0].attrs["href"]}]
+            url = links[-1].attrs["href"]
+            if url.startswith("/browse/users/"):
+                logger.debug(f"Fetching {url}")
+                response = self.request(url)
+                links = response.html.find("#root .item>div>div>a:not(.touchable)")
+                for link in links:
+                    people.append({"name": link.text, "link": link.attrs["href"]})
+            return {"with": people}
+
     @property
     def data_ft(self) -> dict:
         if self._data_ft is not None:
@@ -1012,7 +1114,7 @@ class PostExtractor:
         self._data_ft = {}
         try:
             data_ft_json = self.element.attrs['data-ft']
-            self._data_ft = json.loads(data_ft_json)
+            self._data_ft = demjson.decode(data_ft_json)
         except JSONDecodeError as ex:
             logger.error("Error parsing data-ft JSON: %r", ex)
         except KeyError:
