@@ -89,12 +89,14 @@ class FacebookScraper:
             response = self.get(url)
             if "/watch/" in response.url:
                 video_id = parse_qs(urlparse(response.url).query).get("v")[0]
-                response = self.get(video_id)
+                url = f"story.php?story_fbid={video_id}&id={video_id}&m_entstream_source=video_home&player_suborigin=entry_point&player_format=permalink"
+                logger.debug(f"Fetching {url}")
+                response = self.get(url)
             elem = response.html.find('[data-ft*="top_level_post_id"]', first=True)
             photo_post = False
             if response.html.find("div.msg", first=True):
                 photo_post = True
-                elem = response.html.find("#root", first=True)
+                elem = response.html
             if not elem:
                 logger.warning("No raw posts (<article> elements) were found in this page.")
             else:
@@ -183,22 +185,40 @@ class FacebookScraper:
                 return
 
     def get_profile(self, account, **kwargs) -> Profile:
+        account = account.replace("profile.php?id=", "")
         result = {}
 
         if kwargs.get("allow_extra_requests", True):
             logger.debug(f"Requesting page from: {account}")
             response = self.get(account)
             photo_links = response.html.find("a[href^='/photo.php']")
-            if photo_links:
+            if len(photo_links) == 1:
+                profile_photo = photo_links[0]
+                response = self.get(profile_photo.attrs.get("href"))
+                extractor = PostExtractor(response.html, kwargs, self.get)
+                result["profile_picture"] = extractor.extract_photo_link_HQ(response.html.html)
+            elif len(photo_links) >= 2:
                 cover_photo = photo_links[0]
                 result["cover_photo_text"] = cover_photo.attrs.get("title")
-                response = self.get(cover_photo.attrs.get("href"))
-                extractor = PostExtractor(response.html, kwargs, self.get)
-                result["cover_photo"] = extractor.extract_photo_link_HQ(response.html.html)
+                # Check if there is a cover photo or not
+                if result["cover_photo_text"] is not None:
+                    response = self.get(cover_photo.attrs.get("href"))
+                    extractor = PostExtractor(response.html, kwargs, self.get)
+                    result["cover_photo"] = extractor.extract_photo_link_HQ(response.html.html)
 
-                profile_photo = photo_links[1]
-                response = self.get(profile_photo.attrs.get("href"))
-                result["profile_picture"] = extractor.extract_photo_link_HQ(response.html.html)
+                    profile_photo = photo_links[1]
+                    response = self.get(profile_photo.attrs.get("href"))
+                    result["profile_picture"] = extractor.extract_photo_link_HQ(
+                        response.html.html
+                    )
+                else:
+                    result["cover_photo"] = None
+                    profile_photo = photo_links[0]
+                    response = self.get(profile_photo.attrs.get("href"))
+                    extractor = PostExtractor(response.html, kwargs, self.get)
+                    result["profile_picture"] = extractor.extract_photo_link_HQ(
+                        response.html.html
+                    )
             else:
                 cover_photo = response.html.find(
                     "div[data-sigil='cover-photo']>i.img", first=True
@@ -288,45 +308,63 @@ class FacebookScraper:
 
     def get_page_info(self, page, **kwargs) -> Profile:
         result = {}
-        for post in self.get_posts(page, **kwargs):
-            logger.debug(f"Fetching {post['post_id']}")
-            resp = self.get(post["post_id"])
-            elem = resp.html.find("script[type='application/ld+json']", first=True)
-            if not elem:
-                continue
-            meta = json.loads(elem.text)
-            if meta.get("creator"):
-                result = meta["creator"]
-                result["type"] = result.pop("@type")
-                desc = resp.html.find("meta[name='description']", first=True)
-                if desc:
-                    match = re.search(r'(\d[\d,.]+)', desc.attrs["content"])
-                    if match:
-                        result["likes"] = utils.parse_int(match.groups()[0])
-                try:
-                    for interaction in result.get("interactionStatistic", []):
-                        if interaction["interactionType"] == {
-                            "@type": "http://schema.org/FollowAction"
-                        }:
-                            result["followers"] = interaction["userInteractionCount"]
-                except TypeError as e:
-                    logger.error(e)
-                result.pop("interactionStatistic", None)
-                break
+        desc = None
+
+        if kwargs.get("allow_extra_requests", True):
+            for post in self.get_posts(page, **kwargs):
+                logger.debug(f"Fetching {post['post_id']}")
+                resp = self.get(post["post_id"])
+                elem = resp.html.find("script[type='application/ld+json']", first=True)
+                if not elem:
+                    continue
+                meta = json.loads(elem.text)
+                if meta.get("creator"):
+                    result = meta["creator"]
+                    result["type"] = result.pop("@type")
+                    desc = resp.html.find("meta[name='description']", first=True)
+                    try:
+                        for interaction in result.get("interactionStatistic", []):
+                            if interaction["interactionType"] == {
+                                "@type": "http://schema.org/FollowAction"
+                            }:
+                                result["followers"] = interaction["userInteractionCount"]
+                    except TypeError as e:
+                        logger.error(e)
+                    result.pop("interactionStatistic", None)
+                    break
 
         try:
             about_url = f'/{page}/about/'
             logger.debug(f"Requesting page from: {about_url}")
             resp = self.get(about_url)
             desc = resp.html.find("meta[name='description']", first=True)
-            if desc:
-                logger.debug(desc.attrs["content"])
-                match = re.search(r'(\d[\d,.]+)', desc.attrs["content"])
-                if match:
-                    result["likes"] = utils.parse_int(match.groups()[0])
             result["about"] = resp.html.find('#pages_msite_body_contents', first=True).text
+            cover_photo = resp.html.find("#msite-pages-header-contents i.coverPhoto", first=True)
+            if cover_photo:
+                match = re.search(r"url\('(.+)'\)", cover_photo.attrs["style"])
+                if match:
+                    result["cover_photo"] = utils.decode_css_url(match.groups()[0])
+            profile_photo = resp.html.find("#msite-pages-header-contents img", first=True)
+            if profile_photo:
+                result["profile_photo"] = profile_photo.attrs["src"]
         except Exception as e:
             logger.error(e)
+            try:
+                url = f'/{page}/'
+                logger.debug(f"Requesting page from: {url}")
+                resp = self.get(url)
+                desc = resp.html.find("meta[name='description']", first=True)
+                result["about"] = resp.html.find(
+                    '#pages_msite_body_contents>div>div:nth-child(2)', first=True
+                ).text
+            except Exception as e:
+                logger.error(e)
+        if desc:
+            logger.debug(desc.attrs["content"])
+            match = re.search(r'\..+?(\d[\d,.]+)', desc.attrs["content"])
+            if match:
+                result["likes"] = utils.parse_int(match.groups()[0])
+
         return result
 
     def get_group_info(self, group, **kwargs) -> Profile:
@@ -393,22 +431,26 @@ class FacebookScraper:
         logger.debug(f"Fetching {url}")
         resp = self.get(url)
         more_links = resp.html.find("a[href]", containing="See More")
-        url = more_links[-1].attrs["href"]
-        logger.debug(f"Fetching {url}")
-        resp = self.get(url)
+        if more_links:
+            url = more_links[-1].attrs["href"]
+            logger.debug(f"Fetching {url}")
+            resp = self.get(url)
         items = resp.html.find("div.be")
         results = []
         for item in items:
-            link_elem = item.find("div.bk.bl a", first=True)
+            link_elem = item.find("div.bl a", first=True)
             name = link_elem.text
             link = link_elem.attrs["href"]
             image = item.find("img", first=True).attrs["src"]
-            price = item.find("div.bk.bl")[-1].text
+            price = item.find("div.bl")[-1].text
             result = {"name": name, "link": link, "image": image, "price": price}
             results.append(result)
         return results
 
     def get_group_posts(self, group: Union[str, int], **kwargs) -> Iterator[Post]:
+        self.set_user_agent(
+            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/603.3.8 (KHTML, like Gecko) Version/10.1.2 Safari/603.3.8"
+        )
         iter_pages_fn = partial(iter_group_pages, group=group, request_fn=self.get, **kwargs)
         return self._generic_get_posts(extract_group_post, iter_pages_fn, **kwargs)
 
@@ -435,6 +477,7 @@ class FacebookScraper:
             if (
                 response.url.startswith(FB_MOBILE_BASE_URL)
                 and not response.html.find("script", first=True)
+                and "script" not in response.html.html
                 and self.session.cookies.get("noscript") != "1"
             ):
                 warnings.warn(
@@ -454,7 +497,7 @@ class FacebookScraper:
                     raise exceptions.UnexpectedResponse("Your request couldn't be processed")
                 elif title.text.lower() in temp_ban_titles:
                     raise exceptions.TemporarilyBanned(title.text)
-                elif ">Your Account Has Been Disabled<" in response.html.html:
+                elif ">your account has been disabled<" in response.html.html.lower():
                     raise exceptions.AccountDisabled("Your Account Has Been Disabled")
                 elif (
                     ">We saw unusual activity on your account. This may mean that someone has used your account without your knowledge.<"
@@ -516,7 +559,11 @@ class FacebookScraper:
 
         if "Login approval needed" in response.text or "checkpoint" in response.url:
             raise exceptions.LoginError("Login approval needed")
+        if "The password that you entered is incorrect" in response.text:
+            raise exceptions.LoginError("The password that you entered is incorrect")
         if 'c_user' not in self.session.cookies:
+            with open("login_error.html", "w") as f:
+                f.write(response.text)
             raise exceptions.LoginError("Login unsuccessful")
 
     def is_logged_in(self) -> bool:
