@@ -23,12 +23,15 @@ def iter_pages(account: str, request_fn: RequestFunction, **kwargs) -> Iterator[
         start_url = utils.urljoin(FB_MOBILE_BASE_URL, f'/{account}/posts/')
         try:
             request_fn(start_url)
-        except HTTPError as ex:
+        except Exception as ex:
+            logger.error(ex)
             start_url = utils.urljoin(FB_MOBILE_BASE_URL, f'/{account}/')
     return generic_iter_pages(start_url, PageParser, request_fn, **kwargs)
 
 
-def iter_group_pages(group: Union[str, int], request_fn: RequestFunction, **kwargs) -> Iterator[Page]:
+def iter_group_pages(
+    group: Union[str, int], request_fn: RequestFunction, **kwargs
+) -> Iterator[Page]:
     start_url = kwargs.pop("start_url", None)
 
     if not start_url:
@@ -42,7 +45,9 @@ def iter_photos(account: str, request_fn: RequestFunction, **kwargs) -> Iterator
     return generic_iter_pages(start_url, PhotosPageParser, request_fn, **kwargs)
 
 
-def generic_iter_pages(start_url, page_parser_cls, request_fn: RequestFunction, **kwargs) -> Iterator[Page]:
+def generic_iter_pages(
+    start_url, page_parser_cls, request_fn: RequestFunction, **kwargs
+) -> Iterator[Page]:
     next_url = start_url
 
     request_url_callback = kwargs.get('request_url_callback')
@@ -60,7 +65,9 @@ def generic_iter_pages(start_url, page_parser_cls, request_fn: RequestFunction,
             except HTTPError as e:
                 if e.response.status_code == 500 and retry < RETRY_LIMIT:
                     sleep_duration = retry * 2
-                    logger.debug(f"Caught exception, retry number {retry}. Sleeping for {sleep_duration}s")
+                    logger.debug(
+                        f"Caught exception, retry number {retry}. Sleeping for {sleep_duration}s"
+                    )
                     if retry == (RETRY_LIMIT / 2):
                         logger.debug("Requesting noscript")
                         kwargs["scraper"].set_noscript(True)
@@ -127,7 +134,9 @@ class PageParser:
         match = self.cursor_regex_2.search(self.cursor_blob)
         if match:
             value = match.groups()[0]
-            return utils.unquote(value.encode('utf-8').decode('unicode_escape').replace('\\/', '/')).replace("&amp;", "&")
+            return utils.unquote(
+                value.encode('utf-8').decode('unicode_escape').replace('\\/', '/')
+            ).replace("&amp;", "&")
 
         match = self.cursor_regex_3.search(self.cursor_blob)
         if match:
@@ -168,7 +177,9 @@ class PageParser:
         raw_posts = raw_page.find(selection)
 
         if not raw_posts:
-            logger.warning("No raw posts (<%s> elements) were found in this page." % selection_name)
+            logger.warning(
+                "No raw posts (<%s> elements) were found in this page." % selection_name
+            )
             if logger.isEnabledFor(logging.DEBUG):
                 content = textwrap.indent(
                     raw_page.text,
@@ -181,6 +192,7 @@ class PageParser:
 
         return raw_posts
 
+
 class GroupPageParser(PageParser):
     """Class for parsing a single page of a group"""
 
@@ -203,6 +215,7 @@ class GroupPageParser(PageParser):
     def _parse(self):
         self._parse_html()
 
+
 class PhotosPageParser(PageParser):
     cursor_regex = re.compile(r'href:"(/photos/pandora/[^"]+)"')
     cursor_regex_2 = re.compile(r'href":"(\\/photos\\/pandora\\/[^"]+)"')
@@ -219,4 +232,4 @@ class PhotosPageParser(PageParser):
             match = self.cursor_regex_2.search(self.cursor_blob)
             if match:
                 value = match.groups()[0]
-                return value.encode('utf-8').decode('unicode_escape').replace('\\/', '/')
\ No newline at end of file
+                return value.encode('utf-8').decode('unicode_escape').replace('\\/', '/')
